<?php
/**
 * An interface for seekable iterators which also allow to write to the file
 * in different ways.
 *
 * @author		Roland Haeder <webmaster@ship-simu.org>
 * @version		0.0.0
 * @copyright	Copyright (c) 2007, 2008 Roland Haeder, 2009 - 2014 Core Developer Team
 * @license		GNU GPL 3.0 or any newer version
 * @link		http://www.ship-simu.org
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
interface SeekableWritableFileIterator extends SeekableIterator {
	/**
	 * Seeks to given position
	 *
	 * @param	$seekPosition	Seek position in file
	 * @return	$status			Status of this operation
	 */
	function seek ($seekPosition);

	/**
	 * Size of file stack
	 *
	 * @return	$size	Size (in bytes) of file
	 */
	function size ();

	/**
	 * Reads given amount of bytes from file.
	 *
	 * @param	$bytes	Amount of bytes to read
	 * @return	$data	Data read from file
	 */
	function read ($bytes);

	/**
	 * Analyzes entries in index file. This will count all found (and valid)
	 * entries, mark invalid as damaged and count gaps ("fragmentation"). If
	 * only gaps are found, the file is considered as "virgin" (no entries).
	 *
	 * @return	void
	 */
	function analyzeFile ();

	/**
	 * Checks whether the file header is initialized
	 *
	 * @return	$isInitialized	Whether the file header is initialized
	 */
	function isFileHeaderInitialized ();

	/**
	 * Creates the assigned file
	 *
	 * @return	void
	 */
	function createFileHeader ();

	/**
	 * Pre-allocates file (if enabled) with some space for later faster write access.
	 *
	 * @param	$type	Type of the file
	 * @return	void
	 */
	function preAllocateFile ($type);

	/**
	 * Initializes counter for valid entries, arrays for damaged entries and
	 * an array for gap seek positions. If you call this method on your own,
	 * please re-analyze the file structure. So you are better to call
	 * analyzeFile() instead of this method.
	 *
	 * @return	void
	 */
	function initCountersGapsArray ();

	/**
	 * Getter for header size
	 *
	 * @return	$totalEntries	Size of file header
	 */
	function getHeaderSize ();

	/**
	 * Setter for header size
	 *
	 * @param	$headerSize		Size of file header
	 * @return	void
	 */
	function setHeaderSize ($headerSize);

	/**
	 * Getter for header array
	 *
	 * @return	$totalEntries	Size of file header
	 */
	function getHeader ();

	/**
	 * Setter for header
	 *
	 * @param	$header		Array for a file header
	 * @return	void
	 */
	function setHeader (array $header);

	/**
	 * Updates seekPosition attribute from file to avoid to much access on file.
	 *
	 * @return	void
	 */
	function updateSeekPosition ();

	/**
	 * Getter for total entries
	 *
	 * @return	$totalEntries	Total entries in this file
	 */
	function getCounter ();

	/**
	 * "Getter" for file size
	 *
	 * @return	$fileSize	Size of currently loaded file
	 */
	function getFileSize ();

	/**
	 * Writes data at given position
	 *
	 * @param	$seekPosition	Seek position
	 * @param	$data			Data to be written
	 * @param	$flushHeader	Whether to flush the header (default: flush)
	 * @return	void
	 */
	function writeData ($seekPosition, $data, $flushHeader = TRUE);

	/**
	 * Getter for seek position
	 *
	 * @return	$seekPosition	Current seek position (stored here in object)
	 */
	function getSeekPosition ();
}

// [EOF]
?>
